@@ -1,8 +1,8 @@
 """
-kivy mixin and widgets for to integrate a sideloading server in your app
-========================================================================
+kivy mixin and widgets to integrate a sideloading server in your app
+====================================================================
 
-This namespace portion provides widgets and a mixin class for you main app instance for to easily integrate and control
+This namespace portion provides widgets and a mixin class for you main app instance to easily integrate and control
 the `ae sideloading server <ae.sideloading_server>` into your :mod:`Kivy app <ae.kivy_app>`.
 
 
@@ -32,23 +32,31 @@ To manually start it to offer the APK of the embedding app call the
 
     self.on_sideloading_server_start("", dict())
 
+.. hint:: when you pass the dict with a number in a 'port' key then this number will be used as server listening port.
+
+If no 'port' gets passed then :class:`SideloadingMainAppMixin` will calculate an individual port number from the
+first character of the :attr:`~ae.core.AppBase.app_name` of the app mixing in this class. This is to prevent
+the server socket error `[Errno 98] Address already in use` if two different applications with sideloading are
+running on the same device and want to offer sideloading.
+
 To manually pause the sideloading server call the
 :meth:`#SideloadingMainAppMixin.on_sideloading_server_stop` method passing an empty string and dict::
 
-    self.on_sideloading_server_start("", dict())
+    self.on_sideloading_server_stop("", dict())
+
 
 usage of the sideloading button
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
-This ae namespace portion is additionally providing the `SideloadingButton` flow button widget for to integrate it in
-your Kivy app. This button can be used for to:
+This ae namespace portion is additionally providing the `SideloadingButton` flow button widget to integrate it in
+your Kivy app. This button can be used to:
 
 * start or stop the sideloading server,
 * select a file for sideloading via the :class:`~ae.kivy_file_chooser.FileChooserPopup`.
 * display file info like full file path and file length.
-* display the URL of your sideloading server as QR code for to allow connections from other devices.
+* display the URL of your sideloading server as QR code to allow connections from other devices.
 
-For to optionally integrate this `SideloadingButton` into your app add it to the root layout in your app's main kv file
+To optionally integrate this `SideloadingButton` into your app add it to the root layout in your app's main kv file
 with the `id` `sideloading_button`::
 
     MyRootLayout:
@@ -61,14 +69,14 @@ first check if the `Downloads` folder of the device is containing a APK file for
 sideloading server will be started providing the found APK file.
 
 If the sideloading server is instead already running/active and the user is tapping on the `SideloadingButton` then a
-drop down menu will be shown with options for to (1) display info of the sideloading file, (2) select a new file, (3)
+drop down menu will be shown with options to (1) display info of the sideloading file, (2) select a new file, (3)
 display the sideloading server URL as QR code or (4) stop the sideloading server.
 
 
 dependencies/requirements in buildozer.spec
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
-For to build a Android APK with the kivy sideloading server integrated, make sure that the following external packages
+To build a Android APK with the kivy sideloading server integrated, make sure that the following external packages
 are specified in the `requirements` setting of the `[app]` section of your buildozer.spec file.
 
 * ae.kivy_file_chooser
@@ -95,7 +103,7 @@ requirements = android, hostpython3==3.7.5, python3==3.7.5, kivy==2.0.0,
 sideloading server life cycle
 -----------------------------
 
-For to activate the sideloading server to offer a different file, specify the path (or glob file mask) of the file to be
+To activate the sideloading server to offer a different file, specify the path (or glob file mask) of the file to be
 offered/available via sideloading in the :attr:`~SideloadingMainAppMixin.sideloading_file_mask` attribute and then call
 the method :meth:`~SideloadingMainAppMixin.on_sideloading_server_start`. This method will check if the specified file
 exists and if yes then it will start the sideloading server. If you specify a file mask instead of a concrete file path
@@ -110,9 +118,10 @@ method.
 """
 import os
 
-from typing import Callable, List, Optional
+from typing import Callable, Optional
 
 from kivy.app import App                                                                        # type: ignore
+from kivy.clock import mainthread                                                               # type: ignore
 from kivy.lang import Builder                                                                   # type: ignore
 from kivy.uix.widget import Widget                                                              # type: ignore
 
@@ -121,10 +130,10 @@ from ae.i18n import register_package_translations
 from ae.gui_app import EventKwargsType, id_of_flow, register_package_images, update_tap_kwargs  # type: ignore
 from ae.kivy_app import FlowDropDown, get_txt                                                   # type: ignore
 from ae.sideloading_server import (                                                             # type: ignore
-    DEFAULT_FILE_MASK, server_factory, update_handler_progress, SideloadingServerApp)
+    DEFAULT_FILE_MASK, FILE_COUNT_MISMATCH, server_factory, update_handler_progress, SideloadingServerApp)
 
 
-__version__ = '0.1.4'
+__version__ = '0.1.8'
 
 
 register_package_images()
@@ -165,7 +174,7 @@ Builder.load_string('''\
 
 
 class SideloadingMenuPopup(FlowDropDown):
-    """ dropdown menu for to control sideloading server. """
+    """ dropdown menu to control sideloading server. """
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
 
@@ -214,7 +223,8 @@ class SideloadingMenuPopup(FlowDropDown):
 
 class SideloadingMainAppMixin:
     """ mixin class with default methods for the main app class. """
-    # abstract attributes/properties and methods
+    # abstract attributes/properties and methods provided by the main app instance where this get mixed into
+    app_name: str
     change_app_state: Callable
     change_flow: Callable
     dpo: Callable
@@ -223,8 +233,6 @@ class SideloadingMainAppMixin:
     vpo: Callable
 
     # implemented attributes
-    file_chooser_paths: List[str]                       #: file paths initially created from ae.paths.PATH_PLACEHOLDERS
-
     sideloading_active: tuple                           #: app state flag if sideloading server is running
     sideloading_app: SideloadingServerApp               #: http sideloading server console app
     sideloading_file_ext: str = "."                     #: extension of selected sideloading file
@@ -277,14 +285,17 @@ class SideloadingMainAppMixin:
         """ start the sideloading server.
 
         :param _flow_key:       unused/empty flow key.
-        :param event_kwargs:    event kwargs.
-        :return:                always True for to confirm change of flow id.
+        :param event_kwargs:    event kwargs:
+                                * 'port': TCP/IP server listening port.
+                                * 'tap_widget': button instance that initiated the start of the server.
+        :return:                always True to confirm change of flow id.
         """
+        @mainthread
         def _upd_pr(client_ip: str = "", transferred_bytes: int = -6, total_bytes: int = 0, **kwargs):
             """ update handler attributes for sideloading_app.client_progress and sideloading progress bars. """
             update_handler_progress(
                 client_ip=client_ip, transferred_bytes=transferred_bytes, total_bytes=total_bytes, **kwargs)
-            client_ips = list(self.sideloading_app.client_handlers.keys())
+            client_ips = list(sap.client_handlers.keys())
             if client_ips and total_bytes:
                 fore_last, last = self.sideloading_active
                 if client_ip == client_ips[-1]:
@@ -300,16 +311,22 @@ class SideloadingMainAppMixin:
             self.vpo(f"{pre}stop running sideloading server to restart")
             self.on_sideloading_server_stop("", dict())
 
-        if not self.sideloading_app.start_server(file_mask=self.sideloading_file_mask, progress=_upd_pr, threaded=True):
-            if 'tap_widget' in event_kwargs:    # let user select file if APK is not in downloads folder
+        sap = self.sideloading_app
+
+        sap.set_opt('port', event_kwargs.get('port', 33300 + ord(self.app_name[0])), save_to_config=False)
+
+        err = sap.start_server(file_mask=self.sideloading_file_mask, progress=_upd_pr, threaded=True)
+        if err:
+            self.show_message(err, title=get_txt("server start error"))
+            if FILE_COUNT_MISMATCH in err and 'tap_widget' in event_kwargs:  # let user select APK if match-count != 1
                 self.change_flow(id_of_flow('open', 'file_chooser'),
                                  **update_tap_kwargs(event_kwargs['tap_widget'],
                                                      popup_kwargs=dict(submit_to='sideloading_file_mask')))
             return False
 
-        self.sideloading_file_ext = os.path.splitext(self.sideloading_app.file_path)[1][1:]
+        self.sideloading_file_ext = os.path.splitext(sap.file_path)[1][1:]
         if event_kwargs:    # only display qr code if called from sideloading_button
-            url = self.sideloading_app.server_url()
+            url = sap.server_url()
             self.change_flow(id_of_flow('open', 'qr_displayer'),
                              popup_kwargs=dict(title=url, qr_content=get_txt("sideloading url")))
         self.change_app_state('sideloading_active', (0.0, 0.0))
@@ -321,7 +338,7 @@ class SideloadingMainAppMixin:
 
         :param _flow_key:       unused/empty flow key.
         :param _event_kwargs:   unused event kwargs.
-        :return:                always True for to confirm change of flow id.
+        :return:                always True to confirm change of flow id.
         """
         self.vpo("SideloadingMainAppMixin.on_sideloading_server_stop")
 
